/**
 * TypeSocket options.
 * @interface
 */
export interface TypeSocketOptions {
    /**
     * Maximum amount of retries until a successful connection gets established.
     * Set to 0 to keep retrying forever.
     * Default: 5
     */
    maxRetries?: number;
    /**
     * When set to true the client will try to reconnect when the socket is intentionally closed by either party.
     * Default: false
     */
    retryOnClose?: boolean;
    /**
     * Time between retries. (ms)
     * Default: 500
     */
    retryTime?: number;
}
type TypeSocketConnectionStateChangeEventListener<T> = (this: TypeSocket<T>) => void;
type TypeSocketMessageEventListener<T> = (this: TypeSocket<T>, message: T) => void;
type TypeSocketRawMessageEventListener<T> = (this: TypeSocket<T>, message: string | ArrayBuffer) => void;
type TypeSocketBinaryMessageEventListener<T> = (this: TypeSocket<T>, message: ArrayBuffer) => void;
export declare class TypeSocket<T> {
    private url;
    private socket;
    private retries;
    private options;
    private events;
    /**
     * Creates a new TypeSocket
     * @param url WebSocket server URL
     */
    constructor(url: string, options?: TypeSocketOptions);
    /**
     * Connects to the server.
     *
     * Will automatically retry on failure.
     */
    connect(): void;
    /**
     * Disconnects the connection.
     *
     * When called, TypeSocket will stop retrying, the WebSocket will be closed and both disconnected and permanentlyDisconnected events will be called.
     */
    disconnect(): void;
    /**
     * Sends a JavaScript object of type T to the server.
     * @param data JS object.
     */
    send(data: T): void;
    /**
     * Sends raw data over the socket.
     * @param data Raw data.
     */
    sendRaw(data: string | ArrayBuffer | Blob | ArrayBufferView): void;
    /**
     * Ready state of the socket.
     */
    get readyState(): number;
    /**
     * Adds a listener for a message event.
     * @param eventType Event type. (message)
     * @param listener Listener function.
     */
    on(eventType: 'message', listener: TypeSocketMessageEventListener<T>): void;
    /**
     * Adds a listener for a raw/invalid message event.
     * @param eventType Event type. (message)
     * @param listener Listener function.
     */
    on(eventType: 'rawMessage' | 'invalidMessage', listener: TypeSocketRawMessageEventListener<T>): void;
    /**
     * Adds a listener for a connection event.
     * @param eventType Event type. (connected, disconnected, permanentlyDisconnected)
     * @param listener Listener function.
     */
    on(eventType: 'connected' | 'disconnected' | 'permanentlyDisconnected', listener: TypeSocketConnectionStateChangeEventListener<T>): void;
    /**
     * Adds a listener for a binary message event.
     * @param eventType Event type. (binaryMessage)
     * @param listener Listener function.
     */
    on(eventType: 'binaryMessage', listener: TypeSocketBinaryMessageEventListener<T>): void;
    /**
     * Removes a listener for a message event.
     * @param eventType Event type. (message)
     * @param listener Listener function.
     */
    off(eventType: 'message', listener: TypeSocketMessageEventListener<T>): void;
    /**
     * Removes a listener for a raw/invalid message event.
     * @param eventType Event type. (message)
     * @param listener Listener function.
     */
    off(eventType: 'rawMessage' | 'invalidMessage', listener: TypeSocketRawMessageEventListener<T>): void;
    /**
     * Removes a listener for a connection event.
     * @param eventType Event type. (connected, disconnected, permanentlyDisconnected)
     * @param listener Listener function.
     */
    off(eventType: 'connected' | 'disconnected' | 'permanentlyDisconnected', listener: TypeSocketConnectionStateChangeEventListener<T>): void;
    /**
     * Removes a listener for a binary message event.
     * @param eventType Event type. (binaryMessage)
     * @param listener Listener function.
     */
    off(eventType: 'binaryMessage', listener: TypeSocketBinaryMessageEventListener<T>): void;
    private emit;
    private reconnectAfterTime;
    private reconnect;
    private connected;
    private disconnected;
    private permanentlyDisconnected;
    private message;
}
export {};
