"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeSocket = void 0;
class TypeSocket {
    /**
     * Creates a new TypeSocket
     * @param url WebSocket server URL
     */
    constructor(url, options) {
        this.url = url;
        this.socket = null;
        this.retries = 0;
        this.options = {
            maxRetries: 5,
            retryOnClose: false,
            retryTime: 500,
        };
        this.events = {
            connected: new Set(),
            disconnected: new Set(),
            permanentlyDisconnected: new Set(),
            message: new Set(),
            invalidMessage: new Set(),
            rawMessage: new Set(),
            binaryMessage: new Set(),
        };
        if (options) {
            this.options = {
                ...this.options,
                ...options,
            };
        }
    }
    /**
     * Connects to the server.
     *
     * Will automatically retry on failure.
     */
    connect() {
        if (this.socket) {
            try {
                this.socket.close();
                this.disconnected();
            }
            catch { }
        }
        this.socket = new WebSocket(this.url);
        this.socket.binaryType = 'arraybuffer';
        this.socket.onopen = () => {
            this.connected();
        };
        this.socket.onclose = e => {
            if (e.code === 1000 || (this.options.retryOnClose && this.socket)) {
                this.reconnectAfterTime(this.options.retryTime);
            }
            else {
                this.disconnected();
                this.permanentlyDisconnected();
            }
        };
        this.socket.onmessage = e => {
            this.message(e.data);
        };
        this.socket.onerror = () => {
            this.disconnected();
            this.socket = null;
            this.reconnectAfterTime(this.options.retryTime);
        };
    }
    /**
     * Disconnects the connection.
     *
     * When called, TypeSocket will stop retrying, the WebSocket will be closed and both disconnected and permanentlyDisconnected events will be called.
     */
    disconnect() {
        if (this.socket) {
            try {
                this.socket.onclose = null;
                this.socket.onerror = null;
                this.socket.close();
                this.disconnected();
                this.permanentlyDisconnected();
            }
            catch { }
        }
    }
    /**
     * Sends a JavaScript object of type T to the server.
     * @param data JS object.
     */
    send(data) {
        if (!this.socket)
            return;
        this.socket.send(JSON.stringify(data));
    }
    /**
     * Sends raw data over the socket.
     * @param data Raw data.
     */
    sendRaw(data) {
        if (!this.socket)
            return;
        this.socket.send(data);
    }
    /**
     * Ready state of the socket.
     */
    get readyState() {
        return this.socket ? this.socket.readyState : 0;
    }
    /**
     * Adds a listener for a given event.
     * @param eventType Event type.
     * @param listener Listener function.
     */
    on(eventType, listener) {
        this.events[eventType].add(listener);
    }
    /**
     * Removes a listener for a given event.
     * @param eventType Event type.
     * @param listener Listener function.
     */
    off(eventType, listener) {
        this.events[eventType].delete(listener);
    }
    emit(eventType, ...args) {
        for (const listener of this.events[eventType]) {
            listener.apply(this, args);
        }
    }
    reconnectAfterTime(time = 500) {
        if (this.socket) {
            this.socket.close();
        }
        this.socket = null;
        this.disconnected();
        setTimeout(() => {
            this.reconnect();
        }, time);
    }
    reconnect() {
        this.retries++;
        if (this.options.maxRetries && this.retries > this.options.maxRetries) {
            this.disconnected();
            this.permanentlyDisconnected();
            return;
        }
        this.connect();
    }
    connected() {
        this.retries = 0;
        this.emit('connected');
    }
    disconnected() {
        this.emit('disconnected');
    }
    permanentlyDisconnected() {
        this.emit('permanentlyDisconnected');
    }
    message(data) {
        this.emit('rawMessage', data);
        if (typeof data === 'string') {
            try {
                const json = JSON.parse(data);
                if (json) {
                    this.emit('message', json);
                    return;
                }
            }
            catch { }
        }
        else if (data instanceof ArrayBuffer) {
            this.emit('binaryMessage', data);
        }
        this.emit('invalidMessage', data);
    }
}
exports.TypeSocket = TypeSocket;
